#!/usr/bin/env python

# This application is released under the GNU General Public License 
# v3 (or, at your option, any later version). You can find the full 
# text of the license under http://www.gnu.org/licenses/gpl.txt. 
# By using, editing and/or distributing this software you agree to 
# the terms and conditions of this license.
# Thank you for using free software!
#
#(c) QB89Dragon 2007/8 <hughescih@hotmail.com>
#(c) Whise 2008,2009 <helderfraga@gmail.com>
#
# XDG Menu Functions
# Part of the GnoMenu

import gtk
import backend
import os
import urllib
import xml.dom.minidom
import string
import utils
import gobject
try:
	import gmenu
	print 'gmenu found, using it as default menu parser'
	has_gmenu = True
except:
	has_gmenu = False
	import xdg.Menu
	print 'xdg found, using it as default menu parser'

import xdg.IconTheme
import xdg.Locale
import xdg.BaseDirectory as bd
from xdg.DesktopEntry import *
from Popup_Menu import add_menuitem, add_image_menuitem
import gc
import Globals
try:
	import bookmarks
except:print 'error importing webbookmarks'

try:
	import gio
	isgio = True
except:
	print 'gio not found'
	isgio = False

try:
	INSTALL_PREFIX = open("/etc/gnomenu/prefix").read()[:-1] 
except:
	INSTALL_PREFIX = '/usr'

import gettext

gettext.textdomain('gnomenu')
gettext.install('gnomenu', INSTALL_PREFIX +  '/share/locale')
gettext.bindtextdomain('gnomenu', INSTALL_PREFIX +  '/share/locale')

def _(s):
	return gettext.gettext(s)




class XDGMenu(gobject.GObject):

	__gsignals__ = {
        'changed': (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, ()),
        }
	def __init__(self):
		gobject.GObject.__init__ (self)
		if not os.path.isdir(Globals.AutoStartDirectory):
			os.system('mkdir %s' % Globals.AutoStartDirectory)

		if has_gmenu:
			self.MenuInstance = gmenu.Directory
			self.EntryInstance = gmenu.Entry
		else:
			self.MenuInstance = xdg.Menu.Menu
			self.EntryInstance = xdg.Menu.MenuEntry

		self.AppsFile = None
		self.SetFile = None
		if Globals.OnlyShowFavs: 
			self.BaseMenu = '<Favorites>'
			self.CacheApplications = None
			self.CacheSettings = None
		elif Globals.OnlyShowRecentApps: 
			self.BaseMenu = '<RecentApps>'
			self.CacheApplications = None
			self.CacheSettings = None

		else:
			self.ParseMenus()
			self.BaseMenu = self.CacheApplications

		if isgio:
			self.monitor=gio.volume_monitor_get()
			self.monitor.connect("drive-changed",self.on_drive_changed)

		self.updating = False


		self.recent_manager = gtk.recent_manager_get_default()
		self.recent_manager.connect("changed", self.onRecentChanged)
		self.recents_changed = True
		self.recents = None
		self.ItemComments = {"":""}
		self.Menu = self.BaseMenu
		self.PrevMenu = []
		self.searchresults = 0

		##### This loads the icon cache

		self.IconCachePath = Globals.ConfigDirectory + "/.menuiconcache.xml"
		self.IconNames = []
		self.Icons = []
		self.IconPix = {"":""}
		if Globals.Settings['Show_Thumb']:
			self.thumbnailer = utils.thumbnailengine(Globals.PG_iconsize)
			self.thumbnailer.connect("worklist-finished", lambda m: self.emit('changed'))

		##### looksup icon theme in gtk, we could also use : gconf.client_get_default().get_string('/desktop/gnome/interface/icon_theme')

		self.gtkicontheme = Globals.GtkIconTheme
		self.icontheme = Globals.DefaultIconTheme
		self.old_icontheme = self.icontheme

		##### Check if icon theme is stored in cache#################
		self.LoadIconCache()


		##### Load webbookmarks##################################
		try:
			self.webbookmarker = bookmarks.BookmarksMenu().getBookmarks()
		except:pass
		self.webbookmarker = None

		##### Construct the menu###########################
		self.ConstructMenu()
		gc.collect()

	
	def LoadIconCache(self):
		if os.path.isfile(self.IconCachePath):
			try:
				XMLSettings = xml.dom.minidom.parse(self.IconCachePath)
				self.cacheicontheme = XMLSettings.getElementsByTagName("cache")[0].attributes["IconTheme"].value
			except:
				print "Recoverable Error: Loading of icon cache has failed! This could be due to a corrupted file or read permissions."
				self.cacheicontheme = None
				os.system('rm ' +Globals.ConfigDirectory + "/.menuiconcache.xml")
			if  self.cacheicontheme == self.icontheme:
				print 'icon theme in cache'
				XBase = XMLSettings.childNodes[0].getElementsByTagName("Icon")
				for node in XBase:
					iconname = node.attributes["IconNames"].value
					self.IconNames.append(iconname)
					icon = node.attributes["Icons"].value
					self.Icons.append(icon)
					if os.path.exists(icon):
						pix = gtk.gdk.pixbuf_new_from_file_at_size(icon, Globals.PG_iconsize, Globals.PG_iconsize)
					else:
						pix =	self.geticonfile(gtk.STOCK_MISSING_IMAGE)
					self.IconPix[iconname] = pix
			else:
				print 'icon theme not in cache - restarting cache'
				os.system('rm ' +Globals.ConfigDirectory + "/.menuiconcache.xml")


	def ParseMenus(self):
		"""parse menus using xdg"""

		if has_gmenu:
			self.CacheGmenuApplications = gmenu.lookup_tree(os.environ.get("XDG_MENU_PREFIX", "") + "applications.menu")		
			self.CacheApplications = self.CacheGmenuApplications.get_root_directory()
			if self.CacheApplications == None:
				self.CacheGmenuApplications = gmenu.lookup_tree("gnome-applications.menu")		
				self.CacheApplications = self.CacheGmenuApplications.get_root_directory()

			if self.CacheApplications == None:
				print 'unable to parse the menu'
			else:
				self.CacheGmenuApplications.add_monitor(self.monitor_callback)

			self.CacheGmenuSettings = gmenu.lookup_tree("settings.menu")		
			self.CacheSettings = self.CacheGmenuSettings.get_root_directory()
			if self.CacheSettings == None:
				self.CacheGmenuSettings = gmenu.lookup_tree("gnome-settings.menu")		
				self.CacheSettings = self.CacheGmenuSettings.get_root_directory()
			if self.CacheSettings == None:
				self.CacheGmenuSettings = gmenu.lookup_tree(os.environ.get("XDG_MENU_PREFIX", "") + "settings.menu")		
				self.CacheSettings = self.CacheGmenuSettings.get_root_directory()
			else:
				self.CacheGmenuSettings.add_monitor(self.monitor_callback)
		
			self.MenuInstance = gmenu.Directory
			self.EntryInstance = gmenu.Entry


		else:
			try:
				#standard applications.menu with XDG_MENU_PREFIX that may be null
				self.CacheApplications = xdg.Menu.parse(os.environ.get("XDG_MENU_PREFIX", "") + "applications.menu")
			except:
				print 'Error parsing'
				try:
					# Some distros rename applications.menu without setting XDG_MENU_PREFIX.
					self.CacheApplications = xdg.Menu.parse("gnome-applications.menu")
				except:
					print 'unable to parse the menu'
	
	
			try:
				self.CacheSettings = xdg.Menu.parse("settings.menu")
			except:
				try:
					self.CacheSettings = xdg.Menu.parse("gnome-settings.menu")
				except: 
					try:
						self.CacheSettings = xdg.Menu.parse(os.environ.get("XDG_MENU_PREFIX", "") + "settings.menu")
					except:
						print 'unable to parse the settings menu'

			self.AppsFile = self.CacheApplications.Filename
			self.SetFile = self.CacheSettings.Filename

			if isgio:
				dirs = bd.xdg_data_dirs
				if bd.xdg_data_home not in dirs:
					dirs.append(bd.xdg_data_home)
				for d in dirs:
					self.parentfile = d + '/applications'
					if os.path.isdir(self.parentfile) and os.path.exists(self.parentfile):
						self.current_file1 = gio.File(self.parentfile).monitor_directory()
						self.current_file1.connect_after("changed", self.monitor_callback)
	
				self.current_file = gio.File(self.AppsFile).monitor_file()
				self.current_file.connect_after("changed", self.monitor_callback)

			self.MenuInstance = xdg.Menu.Menu
			self.EntryInstance = xdg.Menu.MenuEntry
	
	

	def Restart(self,menu='all'):
		"""Issues a restart on the menu"""
	
		if Globals.OnlyShowFavs: 
			if menu == 'all' or menu == 'settings':
				menu = 'favorites'

		if Globals.OnlyShowRecentApps: 
			if menu == 'all' or menu == 'settings':
				menu = 'recentapps'
		
		if menu == 'all':
			print 'menu restart Applications'
			self.BaseMenu = self.CacheApplications
			self.Menu = self.BaseMenu
			self.ConstructMenu()
			return True
		elif menu == 'settings':
			print 'menu restart Settings'
			self.BaseMenu = self.CacheSettings
			self.Menu = self.BaseMenu
			self.ConstructMenu()
			return True
		elif menu == 'places':
			print 'menu restart bookmarks'
			self.BaseMenu = "<Bookmarks>"
			self.Menu = self.BaseMenu
			self.ConstructMenu()
			return True
		elif menu == 'auxiliary':
			print 'menu restart auxiliary'
			self.BaseMenu = "<AuxiliaryFunctions>"
			self.Menu = self.BaseMenu
			self.ConstructMenu()
			return True
		elif menu == 'recent':
			print 'menu restart recent files'
			self.BaseMenu = "<RecentItems>"
			self.Menu = self.BaseMenu
			self.ConstructMenu()
		elif menu == 'recentapps':
			print 'menu restart recent apps'
			self.BaseMenu = "<RecentApps>"
			self.Menu = self.BaseMenu
			self.ConstructMenu()
		elif menu == 'webbookmarks':
			print 'menu restart web bookmarks'
			self.BaseMenu = "<WebBookmarks>"
			self.Menu = self.BaseMenu
			self.ConstructMenu()
			return True
		elif menu == 'favorites':
			print 'menu restart favorites'
			self.BaseMenu = "<Favorites>"
			self.Menu = self.BaseMenu
			self.ConstructMenu()
			return True
		elif menu == 'shutdown':
			print 'menu restart shutdown'
			self.BaseMenu = "<Shutdown>"
			self.Menu = self.BaseMenu
			self.ConstructMenu()
			return True
		elif menu == 'previous':
			self.Menu = self.BaseMenu
			print 'menu restart previous'
			self.ConstructMenu()
			return True
		return False

	def getcomment(self,name):
		try:
			comment = self.ItemComments[name]
			return comment
		except: return ''

	def addtomenu(self,names,icons,types,paths,execs):
		"""adds item to menu"""
		if icons == None: icons = gtk.STOCK_MISSING_IMAGE
		self.L_Names.append(names)
		self.L_Icons.append(self.geticonfile(icons))
		self.L_Icons_menu.append(icons)
		self.L_Types.append(types)
		self.L_Paths.append(paths)
		self.L_Execs.append(execs)


	def readMountFile(self, filename):
		"""Reads fstab file"""
		fstab = []
		f = open(filename, 'r')
		for line in f:
			if (not line.isspace() and not line.startswith('#') and not line.lower().startswith('none')) :
				fstabline = line.split()
				if fstabline[1] != 'none' and fstabline[1] != '/proc': fstab.append(fstabline[1])
	
		fstab.sort()
		return fstab


	def monitor_callback(self, monitor=None, Gfile=None, event=None, data=None):
		"""Callback for when directories change"""
		
		if not self.updating:
			self.updating = True
			self.timeout = gobject.timeout_add(1000,self.MenuChanged)



	def MenuChanged(self):
		"""Called 1 second after menu changes"""

		print 'Menu Changed'
		#self.current_file.disconnect_by_func(self.monitor_callback)
		self.ParseMenus()
		if isinstance(self.Menu, self.MenuInstance) and has_gmenu:
			if self.BaseMenu.get_name() ==self.CacheApplications.get_name():self.BaseMenu = self.CacheApplications
			elif self.BaseMenu.get_name() ==self.CacheSettings.get_name():self.BaseMenu = self.CacheSettings
		else:
			if self.BaseMenu ==self.CacheApplications:self.BaseMenu = self.CacheApplications
			elif self.BaseMenu ==self.CacheSettings:self.BaseMenu = self.CacheSettings
		self.emit('changed')
		gc.collect()
		self.updating = False

        def recentSortCriterium(self, item1, item2):
		"""Sort files"""
                return cmp(item2.get_modified(), item1.get_modified())

	
	def onRecentChanged(self, manager):
		"""Recent files have changed"""
		print 'recents changed'
		self.recents_changed = True
		if self.BaseMenu == "<RecentItems>":
			self.emit('changed')

	def on_drive_changed (self,widget,drive):
		"""Bookmarks have changed"""
		print 'drives changed'
		if self.BaseMenu == "<Bookmarks>":
			self.emit('changed')

	def ConstructMenu(self):
		"""Construct the menu"""		
	
		self.L_Names = []
		self.L_Icons = []
		self.L_Types = []
		self.L_Paths = []
		self.L_Execs = []
		self.L_Icons_menu = []
		self.searchresults = 0

		
		#==============================================================
		#AUXILIARY FUNCTIONS MENU
		#==============================================================

		if self.Menu == "<AuxiliaryFunctions>":

			self.addtomenu(_("About Me"),"user-info",5,"","gnome-about-me")
			self.addtomenu(_("Appearance"),"gnome-settings-theme",5,"","gnome-appearance-properties")
			self.addtomenu(_("Menu editor"),"alacarte",6,"","alacarte")
			self.addtomenu(_("Screensaver"),"preferences-desktop-screensaver",5,"","gnome-screensaver-preferences")
			self.addtomenu(_("System Monitor"),"utilities-system-monitor",5,"","gnome-system-monitor")
			self.addtomenu(_("GnoMenu settings utility"),Globals.GraphicsDirectory + "logo.png",7,"",Globals.ProgramDirectory+"GnoMenu-Settings.py")

		#==============================================================
		#FAVORITES ITEMS MENU
		#==============================================================

		elif self.Menu == "<Favorites>":
			client = []
			x = backend.load_setting("favorites")
			if x != None:
				client = x
			try:
				client.sort(key=str.upper)
			except:pass
			
			for elem in client:
				if len(elem.split(':')) >= 3:
					
					self.L_Names.append(elem.split(':')[0])
					try:
						self.L_Icons.append(self.geticonfile(elem.split(':').pop()))
						self.L_Icons_menu.append(elem.split(':').pop())
					except:
						self.L_Icons.append(self.geticonfile(gtk.STOCK_MISSING_IMAGE))
						self.L_Icons_menu.append(gtk.STOCK_MISSING_IMAGE)
					self.L_Paths.append("")
					if len(elem.split(':'))==4:
						self.L_Execs.append(elem.split(':')[1]+':'+elem.split(':')[2])
						if os.path.exists(elem.split(':')[1]+':'+elem.split(':')[2]):
							self.L_Types.append(3)
						else:
							self.L_Types.append(1)
					else:
						self.L_Execs.append(elem.split(':')[1])
						if os.path.exists(elem.split(':')[1]):
							self.L_Types.append(3)
						else:
							self.L_Types.append(1)
			if client == []:
				self.addtomenu(_('No favorites\nUse the mouse right button\nto add or remove favorites'),gtk.STOCK_MISSING_IMAGE,9,"","")


		#==============================================================
		#WEB BOOKMARKS MENU
		#==============================================================

		elif self.Menu == "<WebBookmarks>":

			try:
				if self.webbookmarker == None:
					self.webbookmarker = bookmarks.BookmarksMenu().getBookmarks()
				for item in self.webbookmarker:
					self.addtomenu(item[0],item[3],3,item[1],item[1])	
			except:print 'Error reading web bookmarks'

		elif self.Menu == "<Shutdown>":

			self.addtomenu(_('Shutdown'),'gtk-quit',1,"",Globals.Settings['Shutdown'])
			self.addtomenu(_('Reboot'),'gnome-session-reboot',1,"",Globals.Settings['Restart'])
			self.addtomenu(_('Suspend'),'gnome-session-suspend',1,"",Globals.Settings['Suspend'])	
			self.addtomenu(_('Hibernate'),'gnome-session-hibernate',1,"",Globals.Settings['Hibernate'])	
			#else:
			#	self.addtomenu(_('Shutdown'),'gtk-quit',1,"",'gnome-session-save --shutdown-dialog')
			#	self.addtomenu(_('Reboot'),'gnome-session-reboot',1,"","gnome-session-save --shutdown-dialog")
			#	self.addtomenu(_('Suspend'),'gnome-session-suspend',1,"","gnome-session-save --shutdown-dialog")	
			#	self.addtomenu(_('Hibernate'),'gnome-session-hibernate',1,"","gnome-session-save --shutdown-dialog")	
			self.addtomenu(_('Logout'),'gnome-session-logout',1,"",Globals.Settings['LogoutNow'])
			self.addtomenu(_('Lock Screen'),'system-lock-screen',1,"",Globals.Settings['Lock'])	

		#==============================================================
		#RECENT ITEMS MENU
		#==============================================================
		
		elif self.Menu == "<RecentItems>":

		        #self.recent_manager.set_limit(Globals.RI_numberofitems)
			if self.recents_changed or self.recents == None:
				self.recent = self.recent_manager.get_items()
				self.recents_changed = False	
			self.recent.sort(self.recentSortCriterium)
			x = 0
			for item in self.recent:
				self.addtomenu(item.get_short_name(),self.getthumb(item),3,"",item.get_uri())
				x = x +1
				if x == Globals.RI_numberofitems:break			
			self.addtomenu(_('Clear recent documents'),gtk.STOCK_CLEAR,10,"","")
			self.sorted_list = []

		#==============================================================
		#RECENT APPS MENU
		#==============================================================

		elif self.Menu == "<RecentApps>":

		        #self.recent_manager.set_limit(Globals.RI_numberofitems)
			if self.recents_changed or self.recents == None:
				self.recent = self.recent_manager.get_items()
				self.recents_changed = False	
			self.recent.sort(self.recentSortCriterium)
			x = 0
			for item in self.recent:
				if isgio:
					mime = item.get_mime_type()
					app = gio.app_info_get_default_for_type(mime, 0)
					if app:
						if app.get_name() not in self.L_Names:
							
							self.addtomenu(app.get_name(),self.getgicon(app.get_icon()),1,"",app.get_executable())
							x = x +1
							if x == Globals.RI_numberofitems:break
				else:
					app = item.last_application()
					appinfo = item.get_application_info(app)
					#print appinfo[0].split(' ')[0]
					if app not in self.L_Names:
						
						self.addtomenu(app,appinfo[0].split(' ')[0],1,"",appinfo[0].split(' ')[0])
						x = x +1
						if x == Globals.RI_numberofitems:break
			
	
			self.sorted_list = []

		#==============================================================
		#RECENT BOOKMARKS / PLACES MENU
		#==============================================================

		elif self.Menu == "<Bookmarks>":

			self.addtomenu(_("File System"),"drive-harddisk",3,"/","/")

			if isgio:
			
				self.drives = self.monitor.get_connected_drives()
				for drive in self.drives:
					
					if drive.has_media():
						self.mounts = drive.get_volumes()
						for mount in self.mounts:
							ico = mount.get_icon()
							self.L_Names.append(mount.get_name())
							self.L_Icons.append(self.geticonfile(self.getgicon(ico)))
							self.L_Icons_menu.append(self.getgicon(ico))
							self.L_Types.append(3)
							self.L_Paths.append("")
							try:
								self.L_Execs.append(str(mount.get_mount().get_root().get_uri()).replace("file://",""))
							except:
								self.L_Execs.append("")

			self.addtomenu(_('Home Folder'),"user-home",3,"",Globals.HomeDirectory)
			self.addtomenu(_('Computer'),"computer",3,"",'computer:///')
			self.addtomenu(_('Network'),"network",3,"",'network:///')
			self.addtomenu(_('Trash'),'user-trash',3,"",'trash:///')

			if os.path.isfile(Globals.HomeDirectory + "/.gtk-bookmarks"):
				f = open(Globals.HomeDirectory + "/.gtk-bookmarks", "r")
				data = f.readlines(600)
				f.close()
				f = None
				for i in data:
					
					self.bm = str(i).replace("\n","")
					if self.bm.find(' ') != -1:
						self.folder =  urllib.url2pathname(self.bm[:self.bm.find(' ')])
						self.name = urllib.url2pathname(self.bm[self.bm.find(' ')+1:])
					else: 
						self.folder = self.bm
						self.name = urllib.url2pathname(str(self.bm).split("/").pop()) 
					#print self.folder,self.name
					try:
						if isgio:
							Gfile = gio.File(self.folder)
							tuble = [Gfile, Gfile.query_info("standard::*"), []]
							ico = tuble[1].get_icon()
							self.addtomenu(self.name,self.getgicon(ico),3,"",self.folder)
						else:
							self.addtomenu(self.name,"folder",3,"",self.folder)
					except:pass
		
		#==============================================================
		#XDG PROGRAMS MENU
		#==============================================================

		elif isinstance(self.Menu, self.MenuInstance):


	
			if self.BaseMenu ==self.CacheApplications:
				if Globals.Settings['Shownetandemail'] == 1:
					self.AddInternetButtons()

			if self.BaseMenu ==self.CacheSettings or self.BaseMenu ==self.CacheApplications:
				
				for entry in self.GetMenuEntries(self.Menu):
					
					if isinstance(entry, self.MenuInstance):
						name,icon, path ,comment = self.GetDirProps(entry)		#Folder
						self.addtomenu(name,icon,0,path,"")
						if Globals.Settings['Show_Tips']:
							self.ItemComments[name] = comment
					elif isinstance(entry, self.EntryInstance):	#Application
						name,icon, execute ,comment = self.GetEntryProps(entry)
						if name not in self.L_Names:
							self.addtomenu(name,icon,1,"",execute)
							if Globals.Settings['Show_Tips']:
								self.ItemComments[name] = comment

				if self.Menu ==self.CacheApplications:
					self.AddPlacesButton()
					self.AddSystemButton()
	

		else:
		#==============================================================
		#XDG PROGRAMS MENU (SEARCH)
		#==============================================================
			#self.searchresults = 0
			for i in [self.CacheApplications,self.CacheSettings]:
				for entry in self.GetMenuEntries(i):
					
					if isinstance(entry, self.MenuInstance):		#Folder
						name,icon, path ,comment = self.GetDirProps(entry)		#Folder
						
						if has_gmenu :
							self.SearchMenu(path)
						else:
							self.SearchMenu(i.getMenu(entry.getPath(True)))
	
					elif isinstance(entry, self.EntryInstance):	#Application
						name,icon, execute ,comment = self.GetEntryProps(entry)
						if name.upper().count(self.Menu) != 0:
							self.addtomenu(name,icon,1,"",execute)
							self.searchresults=self.searchresults+1

			if self.searchresults==0:
				self.addtomenu("<separator>","",8,"","")
				self.addtomenu(_("No items matched your search criteria"),"",9,"","")
			else:
				self.addtomenu
				self.addtomenu("<separator>","",8,"","")
				self.addtomenu(_("Found %s results to your search") % str(self.searchresults),"",9,"","")
				
		#Add back button so long as it isn't the base XDG menu
		if isinstance(self.Menu, self.MenuInstance):
			
			if self.Menu !=self.CacheApplications:
				self.AddBackButton()
		else:
			if Globals.MenuTabCount == 0:
				if Globals.OnlyShowFavs == False and Globals.OnlyShowRecentApps == False:
					self.AddBackButton()
				else:
					if self.Menu != "<Favorites>" and self.Menu != "<RecentApps>":
						self.AddBackButton()
		gc.collect()




	def GetMenuEntries(self,menu):

		if has_gmenu:
			return menu.get_contents()
		else:
			return menu.getEntries()

	def GetDirProps(self,entry):

		if has_gmenu:
			if Globals.Settings['Show_Tips']:
				comment = entry.get_comment()
			else:
				comment = ''
			return entry.get_name(), entry.get_icon(),entry,comment
		else:
			if Globals.Settings['Show_Tips']:
				comment = entry.getComment()
			else:
				comment = ''
			return entry.getName(),	entry.getIcon(), entry.getPath(True),comment

	def GetEntryProps(self,entry):

		if has_gmenu:
			if Globals.Settings['Show_Tips']:
				comment = entry.get_comment()
			else:
				comment = ''
			return entry.get_name(), entry.get_icon(),entry.get_exec(),comment
		else:
			entry = entry.DesktopEntry
			if Globals.Settings['Show_Tips']:
				comment = entry.getComment()
			else:
				comment = ''
			return entry.getName(),	entry.getIcon(),entry.getExec(), comment




	def AddPlacesButton(self):
		"""Add Places buttons"""

		a = "Places"
		if _("Places") != None:
			a = _("Places")
		self.addtomenu(a,"drive-harddisk",2,"","")


	def AddSystemButton(self):
		"""Add System buttons"""

		a = "System"
		if _("System") != None:
			a = _("System")
		self.addtomenu(a,"stock_properties",2,"","")


	def SearchMenu(self, folder):

		for entry in self.GetMenuEntries(folder):
			
			if isinstance(entry, self.MenuInstance):		#Folder
				name,icon, path ,comment = self.GetDirProps(entry)		#Folder
				if self.BaseMenu == self.CacheApplications or self.BaseMenu == self.CacheSettings:

					if has_gmenu :
						self.SearchMenu(path)
					else:
						self.SearchMenu(self.BaseMenu.getMenu(entry.getPath(True)))

			elif isinstance(entry, self.EntryInstance):	#Application
				name,icon, execute ,comment = self.GetEntryProps(entry)
				if name.upper().count(self.Menu) != 0:
					self.addtomenu(name,icon,1,"",execute)
					self.searchresults=self.searchresults+1

	def AddInternetButtons(self):
		"""Add Internet and Email buttons"""

		
		# FIXME Languages
		a = "Browse Internet"
		if _("Browse Internet") != None:
			a = _("Browse Internet")
		self.addtomenu(a,"web-browser",1,"",backend.get_default_internet_browser())
		
		
		a = "E-mail"
		if _("E-mail") != None:
			a = _("E-mail")
		self.addtomenu(a,"evolution",1,"",backend.get_default_mail_client())
		#self.addtomenu("<separator>","",8,"","")
		
	def AddBackButton(self):
		"""Add Back buttons"""

		self.addtomenu(_("Back"),"stock_left",2,"","")
		

	def ButtonClick(self,index,event=0):
		"""Callback when buttons are clicked"""

		self.etype = self.L_Types[index]
		if event.type == gtk.gdk.KEY_PRESS:event_button = 1
		elif event.type == gtk.gdk.BUTTON_PRESS:event_button = event.button
		if event_button == 1:
		
			places = "Places"
			if _("Places") != None:
				places = _("Places")
			system = "System"
			if _("System") != None:
				system = _("System")
	
			if self.etype == 0:     				#Subfolder
				self.PrevMenu.append(self.Menu)
				
				if has_gmenu :
					self.Menu = self.L_Paths[index]
				else:
					self.Menu = self.BaseMenu.getMenu(self.L_Paths[index])
				self.ConstructMenu()
			elif self.etype == 1:                   #Application
				print "launching item from application list..."
				self.expl = self.L_Execs[index].split()
				#if last item in list begins with the character "%" then remove it and add it after the ' mark
				self.outcommand = ""
				if "%" in self.expl[len(self.expl)-1]:
					self.outcommand = " " +  self.expl.pop()
				self.expls = string.join(self.expl,' ')			
				if self.expls == 'trash:///':
					self.expls = "xdg-open trash:///"
				elif self.expls == 'network:///':
					self.expls = "xdg-open network:///"
				else:
					if os.path.isdir(self.expls):
						self.expls = "xdg-open '" + self.expls +"'"+self.outcommand
					else:
						self.expls = "/bin/sh -c '" + self.expls +"'"+self.outcommand
				os.system(self.expls+' &')
				self.Restart('previous')
				self.SaveIconCache()
				return 1
			elif self.etype == 2:                              #Menu Back button
				if self.L_Names[index] == _('Back'):
					try:
						self.Menu = self.PrevMenu.pop()
					except:
						self.BaseMenu = self.CacheApplications
						self.Menu=self.BaseMenu
					if self.Menu == self.CacheApplications:
						self.BaseMenu = self.CacheApplications
						self.Menu=self.BaseMenu
					self.ConstructMenu()
				elif self.L_Names[index] == system:
					self.PrevMenu.append(self.Menu)
					self.Restart('settings')
				elif self.L_Names[index] == places:
					self.PrevMenu.append(self.Menu)
					self.Restart('places')
			
			elif self.etype == 3:                                   #Recent Items button (file)
				os.system("xdg-open '"+self.L_Execs[index]+"' &")
				self.Restart('previous')
			elif self.etype == 4:                                                   #Recent Items button (directory)
				os.system("xdg-open '"+self.L_Execs[index]+"' &")
				self.Restart('previous')
			elif self.etype == 5:                                                       #Launch straight file from execlist (NoWait)
				os.system("/bin/sh -c '" + self.L_Execs[index]+"' &")
			elif self.etype == 6:                                                   #Items in the menu have changed
				os.system("/bin/sh -c '" + self.L_Execs[index]+"' &")
			elif self.etype == 7:                                                   #Properties of the menu have changed
				os.system("/bin/sh -c '" + self.L_Execs[index]+"' &")
				Globals.ReloadSettings()
			elif self.etype == 10: #Clear Recent 
				self.recent_manager.purge_items()
				self.Restart('previous')

			#elif self.etype == 9:     				#Subfolder
			#	self.PrevMenu.append(self.Menu)
			#	self.BaseMenu = xdg.Menu.parse("settings.menu")
			#	self.Menu=self.BaseMenu
			#	self.ConstructMenu()
		elif event_button == 3:
			self.m = gtk.Menu()
			#print self.L_Names[index]+ ':' + self.L_Execs[index] + ':' + self.Iconnames[index]
			name = str(self.L_Names[index]).replace(':',' ')
			favlist = backend.load_setting("favorites")
			try:
				add_image_menuitem(self.m, self.L_Icons_menu[index], name, self.dummy,name,self.L_Execs[index],None,self.L_Execs[index])
			except:return
			print self.etype
			if self.etype != 0 and name != _('Back'):
				if self.etype != 3:
					add_menuitem(self.m, "-")
					add_image_menuitem(self.m, gtk.STOCK_DIALOG_AUTHENTICATION, _("Open as Administrator"), self.runasadmin,name,self.L_Execs[index],self.L_Icons_menu[index],self.L_Execs[index])
				if name+ ':' + self.L_Execs[index] + ':' + self.L_Icons_menu[index] not in favlist:
					add_menuitem(self.m, "-")
					add_image_menuitem(self.m, gtk.STOCK_ADD, _("Add to Favorites"), self.addfav,name,self.L_Execs[index],self.L_Icons_menu[index],self.L_Execs[index])
				if name+ ':' + self.L_Execs[index] + ':' + self.L_Icons_menu[index] in favlist:
					add_menuitem(self.m, "-")
					add_image_menuitem(self.m, gtk.STOCK_REMOVE, _("Remove from Favorites"), self.removefav,name,self.L_Execs[index],self.L_Icons_menu[index],self.L_Execs[index])
				add_menuitem(self.m, "-")
				add_image_menuitem(self.m, gtk.STOCK_HOME, _("Create Desktop Shortcut"), self.addshort,name,self.L_Execs[index],self.L_Icons_menu[index],self.L_Execs[index])
				add_menuitem(self.m, "-")
				if (str(name)+'.desktop') in os.listdir(Globals.AutoStartDirectory):
					add_image_menuitem(self.m, gtk.STOCK_REMOVE, _("Remove from System Startup"), self.remove_autostarter,name,self.L_Execs[index],self.L_Icons_menu[index],self.L_Execs[index])
				else:
					add_image_menuitem(self.m, gtk.STOCK_ADD, _("Add to System Startup"), self.create_autostarter,name,self.L_Execs[index],self.L_Icons_menu[index],self.L_Execs[index])	
			self.m.show_all()
			self.m.popup(None, None, None, event.button, event.time)
			

	def run(self,widget,name,execs,ico):
		pass

	def dummy(self,widget,name,execs,ico):
		pass

	def runasadmin(self,widget,name,execs,ico):
		os.system(Globals.Settings['AdminRun'] + ' ' + chr(34) + execs + chr(34) + ' &')


	def addfav(self,widget,name,execs,ico):
		"""Add to Favorites"""
		favlist = backend.load_setting("favorites")
		if name+ ':' + execs + ':' + ico not in favlist:
			favlist.append(name+ ':' + execs + ':' + ico)
			backend.save_setting("favorites",favlist)
		
	def removefav(self,widget,name,execs,ico):
		"""Remove from Favorites"""

		favlist = backend.load_setting("favorites")
		if name+ ':' + execs + ':' + ico in favlist:
			favlist.remove(name+ ':' + execs+ ':' + ico)
			backend.save_setting("favorites",favlist)
			self.emit('changed')

	def create_autostarter (self,widget,name,execs,ico):
		"""Create Autostarter"""

		if not os.path.isdir(Globals.AutoStartDirectory):
			os.system('mkdir %s' % Globals.AutoStartDirectory)
		if (str(name)+'.desktop') not in os.listdir(Globals.AutoStartDirectory):
			self.addshort(widget,name,execs,ico,Globals.AutoStartDirectory)

	def remove_autostarter (self,widget,name,execs,ico):
		"""Remove Autostarter"""

		if not os.path.isdir(Globals.AutoStartDirectory):
			os.system('mkdir %s' % Globals.AutoStartDirectory)
		if (str(name)+'.desktop')  in os.listdir(Globals.AutoStartDirectory):
			os.system('rm %s%s' % (chr(34)+Globals.AutoStartDirectory,(str(name)+'.desktop')+chr(34)))
			
					
	def addshort(self,widget,name,execs,ico,desk='desktop'):
		"""Add Desktop shortcut"""

		if desk =='desktop':desk = self.xdg_dir("XDG_DESKTOP_DIR")
		starter = '%s/%s.desktop' % (desk, name)
		code = ['#!/usr/bin/env xdg-open','[Desktop Entry]']
		code.append('Name=%s' % name)
		code.append('StartupNotify=true')
		code.append('Terminal=false')
		code.append('Version=1.0')
		code.append('Icon=%s' % ico)
		code.append('Type=Application')
		code.append('Exec= %s' % execs)
		code.append('X-GNOME-Autostart-enabled=true')
		#print code
		f = open(starter, 'w')
		if f:
			for l in code:
				f.write(l + '\n')
			f.close()
			
		
	def xdg_dir(self,key):
		"""http://www.freedesktop.org/wiki/Software/xdg-user-dirs"""
		try:
			user_dirs = bd.xdg_config_home + '/user-dirs.dirs'
		except:
			user_dirs = os.path.expanduser("~/.config/user-dirs.dirs")
		if os.path.exists(user_dirs):
			f = open(user_dirs, "r")
			for line in f.readlines():
				if line.startswith(key):
					return os.path.expandvars(line[len(key)+2:-2])
		return False


	def Icon_change(self):
		"""Icons have changed"""

		print 'icons changed'
		self.icontheme = gtk.settings_get_default().get_property("gtk-icon-theme-name")
		self.gtkicontheme = gtk.icon_theme_get_default()
		if self.icontheme != self.old_icontheme:
				
			self.IconNames = []
			self.Icons = []
			self.IconPix.clear()
			self.old_icontheme = self.icontheme	



	def getgicon(self,gico):
		"""Returns gio icon"""
		try:
			names = self.gtkicontheme.lookup_by_gicon(gico,Globals.PG_iconsize,gtk.ICON_LOOKUP_USE_BUILTIN);
			if names:
				return names.get_filename()
		except:pass
		try:
			names = gico.get_names()
			for x in range(0,len(names)):
				if self.gtkicontheme.has_icon(names[x]):
					return names[x]
		except:pass
		return 'gtk-execute'

	def addicontomen(self,icon,iconpath,pix):
		"""Adds icons to cache"""
		self.IconNames.append(icon)
		self.Icons.append(iconpath)
		self.IconPix[icon] = pix
		gc.collect()
		

	def getthumb(self,item):
		path = urllib.url2pathname(item.get_uri().replace('file://',''))
		if Globals.Settings['Show_Thumb']:
			if not os.path.exists(path): 
				return item.get_icon(Globals.PG_iconsize)
			if isgio:
				self.gfile = gio.File(path)
				self.info = self.gfile.query_info(gio.FILE_ATTRIBUTE_THUMBNAIL_PATH, gio.FILE_QUERY_INFO_NONE)
				thumbfile = self.info.get_attribute_as_string(gio.FILE_ATTRIBUTE_THUMBNAIL_PATH)
				if thumbfile:
					if os.path.isfile(thumbfile): return thumbfile
					else:return item.get_icon(Globals.PG_iconsize)
				else:return item.get_icon(Globals.PG_iconsize)



			else:
				icon, isthumbnail = self.thumbnailer.CheckWorkList(path)
				if not icon:
					self.thumbnailer.AddToWorkList(path, item.get_mime_type())
					return item.get_icon(Globals.PG_iconsize)
				else:
					if isthumbnail:
						return icon
					else: return item.get_icon(Globals.PG_iconsize)
		else:
			return item.get_icon(Globals.PG_iconsize)



	def geticonfile(self,icon):
		"""Returns system icon paths"""

		# If icons is pixbuf we do nothing
		if isinstance(icon, gtk.gdk.Pixbuf):
		#if type(icon).__name__ == 'Pixbuf':
			return icon

		# Reload icons if icons have changed since last restart
		if self.icontheme != self.old_icontheme:
				
			self.IconNames = []
			self.Icons = []
			self.IconPix.clear()
			self.old_icontheme = self.icontheme	

		# If icon is in memory return icons, else get icon

		if icon in self.IconNames:

			return self.IconPix[icon]#gtk.gdk.pixbuf_new_from_file_at_size(icon, Globals.PG_iconsize, Globals.PG_iconsize)
		else:

			# lockup icon in gtk icon theme
			
			self.iconinfo = self.gtkicontheme.lookup_icon(str(icon),Globals.PG_iconsize,gtk.ICON_LOOKUP_USE_BUILTIN);
			
			if self.iconinfo:
				iconpath = self.iconinfo.get_filename()
				if iconpath:
					if os.path.isfile(iconpath):
						pix = gtk.gdk.pixbuf_new_from_file_at_size(iconpath, Globals.PG_iconsize, Globals.PG_iconsize)
						self.addicontomen(icon,iconpath,pix)
						gc.collect()
						return pix
				else:
					pix = self.iconinfo.get_builtin_pixbuf()
					if pix:
						return pix
			
			# lockup icon in xdg icon theme
			try:
				iconpath = xdg.IconTheme.getIconPath(icon, Globals.PG_iconsize, self.icontheme, ["png", "xpm", "svg"])
			except: iconpath = None # above receives errors sometimes
			if iconpath:
				if os.path.exists(iconpath):
					pix = gtk.gdk.pixbuf_new_from_file_at_size(iconpath, Globals.PG_iconsize, Globals.PG_iconsize)	
					self.addicontomen(icon,iconpath,pix)
					gc.collect()
					return pix

			# if icons not found return gtk missing image
			self.iconinfo = self.gtkicontheme.lookup_icon(gtk.STOCK_MISSING_IMAGE,Globals.PG_iconsize,gtk.ICON_LOOKUP_USE_BUILTIN);
			if self.iconinfo:
				iconpath = self.iconinfo.get_filename()
				if iconpath:
					if os.path.exists(iconpath):
						pix = gtk.gdk.pixbuf_new_from_file_at_size(iconpath, Globals.PG_iconsize, Globals.PG_iconsize)
						self.addicontomen(gtk.STOCK_MISSING_IMAGE,iconpath,pix)
						gc.collect()
						return pix
				else:
					pix = self.iconinfo.get_builtin_pixbuf()
					if pix:
						return pix

			# if all else fails return gnomenu own missing image, should never append...

			iconpath = Globals.BrokenImage
			pix = gtk.gdk.pixbuf_new_from_file_at_size(Globals.BrokenImage, Globals.PG_iconsize, Globals.PG_iconsize)
			self.addicontomen(icon,iconpath,pix)
			gc.collect()
			return pix


	def CallSpecialMenu(self,command,data=None):
		"""Call was placed on the menu"""

		command = int(command)
		a = str(self.Menu)
		if command==0:	#Back button
			if self.PrevMenu:
				self.Menu = self.PrevMenu.pop()
				self.ConstructMenu()
		elif command==1:	#Go to base menu and reload it
			self.PrevMenu = []
			self.Restart()
		elif command==2:	#Go to unmount drives menu
			if a != '<RecentApps>':
				self.PrevMenu.append(self.Menu)
				self.Restart('recentapps')
		elif command==3:	#Auxiliary functions menu
			if a != '<AuxiliaryFunctions>':
				self.PrevMenu.append(self.Menu)
				self.Restart('auxiliary')
		elif command==4:	#Recent Items Menu
			if a != '<RecentItems>':
				self.PrevMenu.append(self.Menu)
				self.Restart('recent')
		elif command==5:	#Search
			if data!="":
				self.PrevMenu = [self.BaseMenu]
				self.Menu=data.upper()
				self.ConstructMenu()
			else:
				self.PrevMenu = []
				self.Menu=self.BaseMenu
				self.ConstructMenu()
		elif command==6:	#Launch first item in search list
			if self.searchresults!=0:
				if Globals.Settings['Shownetandemail'] == 1:
					self.ButtonClick(3)
				else:
					self.ButtonClick(0)
		elif command==7:	#Recent Items Menu
			if a != '<Favorites>':
				self.PrevMenu.append(self.Menu)
				self.Restart('favorites')
		elif command==8:	#Recent Items Menu
			if a != '<Bookmarks>':
				self.PrevMenu.append(self.Menu)
				self.Restart('places')
		elif command==9:	#Recent Items Menu
			if a != '<Shutdown>':
				self.PrevMenu.append(self.Menu)
				self.Restart('shutdown')
		elif command==10:	#Recent Items Menu
			if a != '<WebBookmarks>':
				self.PrevMenu.append(self.Menu)
				self.Restart('webbookmarks')


	def SaveIconCache(self):
		print "Saving Icon Cache..."
		#Save Imagecache
		if os.path.exists(self.IconCachePath):
			f = open(self.IconCachePath,"r").read()
		else: f = None
		try:
			XMLSettings = xml.dom.minidom.Document()
			XBase = XMLSettings.createElement("cache")
			XBase.setAttribute("IconTheme", self.icontheme)
			XMLSettings.appendChild(XBase)
			for icon in range(0,len(self.IconNames)):
				node = XMLSettings.createElement("Icon")
				node.setAttribute("IconNames", self.IconNames[icon])
				node.setAttribute("Icons", self.Icons[icon])
				XBase.appendChild(node)
			#Now write it to the file
			file = open(self.IconCachePath,"w")
			XMLSettings.writexml(file, "    ", "", "\n", "UTF-8")
			XMLSettings.unlink()
			print "Icon cache successfully written to user's home directory"		
		except:
			print "Error - Icon cache saving error"
			if f:
				file = open(self.IconCachePath,"w")
				file.write(f)
				file.close()
	def destroy(self):
		self.SaveIconCache()
